package reborncore.common.tile;

import mcp.MethodsReturnNonnullByDefault;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.Packet;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.world.World;
import org.apache.commons.lang3.ArrayUtils;
import reborncore.api.recipe.IRecipeCrafterProvider;
import reborncore.api.tile.IContainerProvider;
import reborncore.api.tile.IInventoryProvider;
import reborncore.client.gui.BaseSlot;
import reborncore.common.blocks.BlockMachineBase;
import reborncore.common.container.RebornContainer;
import reborncore.common.packets.PacketHandler;
import reborncore.common.recipes.RecipeCrafter;
import reborncore.common.util.Inventory;
import scala.xml.dtd.impl.Base;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Optional;

public class TileMachineBase extends TileEntity implements ITickable, IInventory, ISidedInventory {

	public void syncWithAll() {
		if (!field_145850_b.field_72995_K) {
			PacketHandler.sendPacketToAllPlayers(func_189518_D_(), field_145850_b);
		}
	}

	public void syncWithNearestPlayers() {
		if (!field_145850_b.field_72995_K) {
			SPacketUpdateTileEntity packet = func_189518_D_();
			for (EntityPlayer player : field_145850_b.field_73010_i) {
				if (player.func_174818_b(func_174877_v()) < 64 * 64) {
					((EntityPlayerMP) player).field_71135_a.func_147359_a(packet);
				}
			}
		}
	}

	@Nonnull
	@Override
	public SPacketUpdateTileEntity func_189518_D_() {
		return new SPacketUpdateTileEntity(func_174877_v(), func_145832_p(), func_189515_b(new NBTTagCompound()));
	}

	@Override
	public NBTTagCompound func_189517_E_() {
		NBTTagCompound compound = super.func_189515_b(new NBTTagCompound());
		func_189515_b(compound);
		return compound;
	}

	@Override
	public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity packet) {
		func_145839_a(packet.func_148857_g());
	}

	@Override
	public void func_73660_a() {
		updateEntity();
		if (getCrafterForTile().isPresent()) {
			getCrafterForTile().get().updateEntity();
		}
	}

	@Deprecated
	public void updateEntity() {}

	public int getFacingInt() {
		Block block = field_145850_b.func_180495_p(field_174879_c).func_177230_c();
		if (block instanceof BlockMachineBase) {
			return ((BlockMachineBase) block).getFacing(field_145850_b.func_180495_p(field_174879_c)).func_176745_a();
		}
		return 0;
	}

	public EnumFacing getFacingEnum() {
		Block block = field_145850_b.func_180495_p(field_174879_c).func_177230_c();
		if (block instanceof BlockMachineBase) {
			return ((BlockMachineBase) block).getFacing(field_145850_b.func_180495_p(field_174879_c));
		}
		return null;
	}

	public void setFacing(EnumFacing enumFacing) {
		Block block = field_145850_b.func_180495_p(field_174879_c).func_177230_c();
		if (block instanceof BlockMachineBase) {
			((BlockMachineBase) block).setFacing(enumFacing, field_145850_b, field_174879_c);
		}
	}

	public boolean isActive() {
		Block block = field_145850_b.func_180495_p(field_174879_c).func_177230_c();
		if (block instanceof BlockMachineBase) {
			return field_145850_b.func_180495_p(field_174879_c).func_177229_b(BlockMachineBase.ACTIVE);
		}
		return false;
	}

	// This stops the tile from getting cleared when the state is
	// updated(rotation and on/off)
	@Override
	public boolean shouldRefresh(World world, BlockPos pos, IBlockState oldState, IBlockState newSate) {
		return oldState.func_177230_c() != newSate.func_177230_c();
	}

	protected Optional<Inventory> getInventoryForTile() {
		if (this instanceof IInventoryProvider) {
			IInventoryProvider inventory = (IInventoryProvider) this;
			if (inventory.getInventory() == null) {
				return Optional.empty();
			}
			return Optional.of(inventory.getInventory());
		} else {
			return Optional.empty();
		}
	}

	protected Optional<RecipeCrafter> getCrafterForTile() {
		if (this instanceof IRecipeCrafterProvider) {
			IRecipeCrafterProvider crafterProvider = (IRecipeCrafterProvider) this;
			if (crafterProvider.getRecipeCrafter() == null) {
				return Optional.empty();
			}
			return Optional.of(crafterProvider.getRecipeCrafter());
		} else {
			return Optional.empty();
		}
	}

	protected Optional<RebornContainer> getContainerForTile() {
		if (this instanceof IContainerProvider) {
			IContainerProvider containerProvider = (IContainerProvider) this;
			if (containerProvider.getContainer() == null) {
				return Optional.empty();
			}
			return Optional.of(containerProvider.getContainer());
		} else {
			return Optional.empty();
		}
	}

	@Override
	public void func_145839_a(NBTTagCompound tagCompound) {
		super.func_145839_a(tagCompound);
		if (getInventoryForTile().isPresent()) {
			getInventoryForTile().get().readFromNBT(tagCompound);
		}
		if (getCrafterForTile().isPresent()) {
			getCrafterForTile().get().readFromNBT(tagCompound);
		}
	}

	@Override
	public NBTTagCompound func_189515_b(NBTTagCompound tagCompound) {
		super.func_189515_b(tagCompound);
		if (getInventoryForTile().isPresent()) {
			getInventoryForTile().get().writeToNBT(tagCompound);
		}
		if (getCrafterForTile().isPresent()) {
			getCrafterForTile().get().writeToNBT(tagCompound);
		}
		return tagCompound;
	}

	//Inventory Start
	@Override
	public int func_70302_i_() {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_70302_i_();
		}
		return 0;
	}

	@Override
	public ItemStack func_70301_a(int index) {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_70301_a(index);
		}
		return null;
	}

	@Override
	public ItemStack func_70298_a(int index, int count) {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_70298_a(index, count);
		}
		return null;
	}

	@Override
	public ItemStack func_70304_b(int index) {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_70304_b(index);
		}
		return null;
	}

	@Override
	public void func_70299_a(int index, ItemStack stack) {
		if (getInventoryForTile().isPresent()) {
			getInventoryForTile().get().func_70299_a(index, stack);
		}
	}

	@Override
	public int func_70297_j_() {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_70297_j_();
		}
		return 0;
	}

	@Override
	public boolean func_70300_a(EntityPlayer player) {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_70300_a(player);
		}
		return false;
	}

	@Override
	public void func_174889_b(EntityPlayer player) {
		if (getInventoryForTile().isPresent()) {
			getInventoryForTile().get().func_174889_b(player);
		}
	}

	@Override
	public void func_174886_c(EntityPlayer player) {
		if (getInventoryForTile().isPresent()) {
			getInventoryForTile().get().func_174886_c(player);
		}
	}

	@Override
	public boolean func_94041_b(int index, ItemStack stack) {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_94041_b(index, stack);
		}
		return false;
	}

	@Override
	public int func_174887_a_(int id) {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_174887_a_(id);
		}
		return 0;
	}

	@Override
	public void func_174885_b(int id, int value) {
		if (getInventoryForTile().isPresent()) {
			getInventoryForTile().get().func_174885_b(id, value);
		}
	}

	@Override
	public int func_174890_g() {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_174890_g();
		}
		return 0;
	}

	@Override
	public void func_174888_l() {
		if (getInventoryForTile().isPresent()) {
			getInventoryForTile().get().func_174888_l();
		}
	}

	@Override
	public String func_70005_c_() {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_70005_c_();
		}
		return null;
	}

	@Override
	public boolean func_145818_k_() {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_145818_k_();
		}
		return false;
	}

	@Override
	public ITextComponent func_145748_c_() {
		if (getInventoryForTile().isPresent()) {
			return getInventoryForTile().get().func_145748_c_();
		}
		return null;
	}

	@Override
	public int[] func_180463_a(EnumFacing side) {
		if (getContainerForTile().isPresent()) {
			RebornContainer container = getContainerForTile().get();
			ArrayList<Integer> intList = new ArrayList<>();
			for (int i = 0; i < container.slotMap.size(); i++) {
				intList.add(i);
			}
			int[] intArr = ArrayUtils.toPrimitive(intList.toArray(new Integer[intList.size()]));
			return intArr;
		}
		return new int[0];
	}

	@Override
	public boolean func_180462_a(int index, ItemStack itemStackIn, EnumFacing direction) {
		if (getContainerForTile().isPresent()) {
			RebornContainer container = getContainerForTile().get();
			if (container.slotMap.containsKey(index)) {
				Slot slot = container.slotMap.get(index);
				if (slot.func_75214_a(itemStackIn)) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public boolean func_180461_b(int index, ItemStack stack, EnumFacing direction) {
		if (getContainerForTile().isPresent()) {
			RebornContainer container = getContainerForTile().get();
			if (container.slotMap.containsKey(index)) {
				BaseSlot slot = container.slotMap.get(index);
				if (slot.canWorldBlockRemove()) {
					return true;
				}
			}
		}
		return false;
	}
	//Inventory end
}
