package reborncore.common.util;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;

public class ItemNBTHelper
{

	/**
	 * Checks if an ItemStack has a Tag Compound
	 **/
	public static boolean detectNBT(ItemStack stack)
	{
		return stack.func_77942_o();
	}

	/**
	 * Tries to initialize an NBT Tag Compound in an ItemStack, this will not do
	 * anything if the stack already has a tag compound
	 **/
	public static void initNBT(ItemStack stack)
	{
		if (!detectNBT(stack))
			injectNBT(stack, new NBTTagCompound());
	}

	/**
	 * Injects an NBT Tag Compound to an ItemStack, no checks are made
	 * previously
	 **/
	public static void injectNBT(ItemStack stack, NBTTagCompound nbt)
	{
		stack.func_77982_d(nbt);
	}

	/**
	 * Gets the NBTTagCompound in an ItemStack. Tries to init it previously in
	 * case there isn't one present
	 **/
	public static NBTTagCompound getNBT(ItemStack stack)
	{
		initNBT(stack);
		return stack.func_77978_p();
	}

	// SETTERS
	// ///////////////////////////////////////////////////////////////////

	public static void setBoolean(ItemStack stack, String tag, boolean b)
	{
		getNBT(stack).func_74757_a(tag, b);
	}

	public static void setByte(ItemStack stack, String tag, byte b)
	{
		getNBT(stack).func_74774_a(tag, b);
	}

	public static void setShort(ItemStack stack, String tag, short s)
	{
		getNBT(stack).func_74777_a(tag, s);
	}

	public static void setInt(ItemStack stack, String tag, int i)
	{
		getNBT(stack).func_74768_a(tag, i);
	}

	public static void setLong(ItemStack stack, String tag, long l)
	{
		getNBT(stack).func_74772_a(tag, l);
	}

	public static void setFloat(ItemStack stack, String tag, float f)
	{
		getNBT(stack).func_74776_a(tag, f);
	}

	public static void setDouble(ItemStack stack, String tag, double d)
	{
		getNBT(stack).func_74780_a(tag, d);
	}

	public static void setCompound(ItemStack stack, String tag, NBTTagCompound cmp)
	{
		if (!tag.equalsIgnoreCase("ench")) // not override the enchantments
			getNBT(stack).func_74782_a(tag, cmp);
	}

	public static void setString(ItemStack stack, String tag, String s)
	{
		getNBT(stack).func_74778_a(tag, s);
	}

	public static void setList(ItemStack stack, String tag, NBTTagList list)
	{
		getNBT(stack).func_74782_a(tag, list);
	}

	// GETTERS
	// ///////////////////////////////////////////////////////////////////

	public static boolean verifyExistance(ItemStack stack, String tag)
	{
		return stack != null && getNBT(stack).func_74764_b(tag);
	}

	public static boolean getBoolean(ItemStack stack, String tag, boolean defaultExpected)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_74767_n(tag) : defaultExpected;
	}

	public static byte getByte(ItemStack stack, String tag, byte defaultExpected)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_74771_c(tag) : defaultExpected;
	}

	public static short getShort(ItemStack stack, String tag, short defaultExpected)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_74765_d(tag) : defaultExpected;
	}

	public static int getInt(ItemStack stack, String tag, int defaultExpected)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_74762_e(tag) : defaultExpected;
	}

	public static long getLong(ItemStack stack, String tag, long defaultExpected)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_74763_f(tag) : defaultExpected;
	}

	public static float getFloat(ItemStack stack, String tag, float defaultExpected)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_74760_g(tag) : defaultExpected;
	}

	public static double getDouble(ItemStack stack, String tag, double defaultExpected)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_74769_h(tag) : defaultExpected;
	}

	/**
	 * If nullifyOnFail is true it'll return null if it doesn't find any
	 * compounds, otherwise it'll return a new one.
	 **/
	public static NBTTagCompound getCompound(ItemStack stack, String tag, boolean nullifyOnFail)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_74775_l(tag)
				: nullifyOnFail ? null : new NBTTagCompound();
	}

	public static String getString(ItemStack stack, String tag, String defaultExpected)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_74779_i(tag) : defaultExpected;
	}

	public static NBTTagList getList(ItemStack stack, String tag, int objtype, boolean nullifyOnFail)
	{
		return verifyExistance(stack, tag) ? getNBT(stack).func_150295_c(tag, objtype)
				: nullifyOnFail ? null : new NBTTagList();
	}
}
